﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Dapfor.Wpf.Controls;

namespace Dapfor.Wpf.Grid.Demo.Examples.Basics.CustomFilters
{
    /// <summary>
    ///     Interaction logic for RatingFilter.xaml
    /// </summary>
    public partial class RatingFilter : UserControl
    {
        private bool _suspendUpdates;

        public RatingFilter()
        {
            InitializeComponent();
        }

        protected override void OnPreviewKeyDown(KeyEventArgs e)
        {
            base.OnPreviewKeyDown(e);
            if (e.Key == Key.Escape)
            {
                Column.CloseFilter();
            }
        }

        #region ColumnProperty

        /// <summary>
        ///     Identifies the <see cref="Column" /> dependency property.
        /// </summary>
        public static readonly DependencyProperty ColumnProperty = DependencyProperty.Register("Column", typeof (Column), typeof (RatingFilter), new FrameworkPropertyMetadata(null, OnChangeColumn));

        private static void OnChangeColumn(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            //Restore persisted data
            var control = (RatingFilter)d;
            if (control.Column != null && control.Column.Tag is int)
            {
                control._suspendUpdates = true;
                control.Rating.RatingValue = (int)control.Column.Tag;
                control._suspendUpdates = false;
            }
        }

        /// <summary>
        ///     The actually filtering <see cref="Column" />.
        /// </summary>
        public Column Column
        {
            get { return (Column) GetValue(ColumnProperty); }
            set { SetValue(ColumnProperty, value); }
        }

        #endregion

        private void Rating_RatingChanged(object sender, RoutedEventArgs e)
        {
            if (!_suspendUpdates && Column != null)
            {
                Column.Tag = Rating.RatingValue;
                Column.Filter = new FilterRules(Column, Rating.RatingValue);
            }
        }

        private class FilterRules : IFilter
        {
            private readonly Column _column;
            private readonly int _value;

            public FilterRules(Column column, int value)
            {
                _column = column;
                _value = value;
            }

            public bool IsFiltered(Row row)
            {
                var cell = row[_column.Id];
                var v = cell != null ? cell.Value : null;
                return (v as int?) > _value;
            }
        }
    }
}