﻿using System;
using System.Globalization;
using System.Windows.Data;
using System.Windows.Threading;
using Dapfor.Wpf.Grid.Demo.Internal;
using Dapfor.Wpf.Grid.Demo.Utils;

namespace Dapfor.Wpf.Grid.Demo.Examples.Performance.MemoryConsumption
{
    /// <summary>
    /// Interaction logic for MemoryConsumptionDemo.xaml
    /// </summary>
    //[Example("Performance", "Memory consumption", "Big feature!!!")]
    public partial class MemoryConsumptionDemo : IDisposable
    {
        private readonly DispatcherTimer _timer;

        public MemoryConsumptionDemo()
        {
            InitializeComponent();

            tbPrivateBytes.Text = string.Format("{0:n2} Mb", ((double)MemoryHelper.PrivateBytes) / (1024 * 1024));

            _timer = new DispatcherTimer();
            _timer.Interval = TimeSpan.FromSeconds(1);
            _timer.Tick += delegate
            {
                tbPrivateBytes.Text = string.Format("{0:n2} Mb", ((double)MemoryHelper.PrivateBytes) / (1024 * 1024));
            };

            _timer.Start();
        }

        private void OnBtnAddRowsClick(object sender, System.Windows.RoutedEventArgs e)
        {
            int count;
            if (!string.IsNullOrEmpty(tbAddRows.Text) && int.TryParse(tbAddRows.Text, NumberStyles.Number, CultureInfo.InvariantCulture, out count) && count >= 0)
            {
                Random random = new Random();
                //Force garbage collection before adding items
                GC.Collect();
                long totalMemory = GC.GetTotalMemory(true);

                for (int i = 0; i < count; ++i)
                {
                    DataObject dataObject = new DataObject(random);
                    grid.Rows.Add(dataObject);
                }

                //Force garbage collection after items have been added
                GC.Collect();
                long totalMemory2 = GC.GetTotalMemory(true);


                int bytesPerRow = (int)((totalMemory2 - totalMemory) / count);
                tbStat.Text = string.Format("Added {0} rows. Consumption: {1:n2} Mb, {2} bytes/row",
                                                       count,
                                                       ((double)(totalMemory2 - totalMemory)) / (1024 * 1024),
                                                       bytesPerRow);

                tbRowCount.Text = string.Format("{0}", grid.Rows.Count);
            }
        }

        private void OnBtnRemoveRowsClick(object sender, System.Windows.RoutedEventArgs e)
        {
            int count;
            if (!string.IsNullOrEmpty(tbRemoveRows.Text) && int.TryParse(tbRemoveRows.Text, NumberStyles.Number, CultureInfo.InvariantCulture, out count) && count >= 0)
            {
                count = Math.Min(count, grid.Rows.Count);
                if (count > 0)
                {
                    //Force garbage collection before adding items
                    GC.Collect();
                    long totalMemory = GC.GetTotalMemory(true);

                    for (int i = 0; i < count; ++i)
                    {
                        grid.Rows.RemoveAt(grid.Rows.Count - 1);
                    }


                    //Force garbage collection after items have been added
                    GC.Collect();
                    long totalMemory2 = GC.GetTotalMemory(true);


                    int bytesPerRow = (int)((totalMemory - totalMemory2) / count);
                    tbStat.Text = string.Format("Removed {0} rows. Collected: {1:n2} Mb, {2} bytes/row",
                                                           count,
                                                           ((double)(totalMemory - totalMemory2)) / (1024 * 1024),
                                                           bytesPerRow);

                    tbRowCount.Text = string.Format("{0}", grid.Rows.Count);
                }
            }
        }

        public void Dispose()
        {
            grid.Rows.Clear();
            _timer.Stop();

            BindingOperations.ClearAllBindings(this);
        }
    }
}